import socket
import tkinter as tk
import time
import webbrowser
import tkinter.font as tkFont

class IPEntryWindow:
    def __init__(self, master, callback):
        self.error = False
        self.master = master
        self.callback = callback
        master.title("Enter IP Address")
        master.geometry("420x530")
        master.config(bg = "black")
        
        self.link1 = tk.Label(master = master, text = "AeroAudio.eu", bg = "black", fg = "aqua", cursor = "hand2")
        self.link1.pack(side = tk.TOP)
        font = tkFont.Font(self.link1, self.link1.cget("font"))
        font.configure(underline = True)
        self.link1.configure(font = font)
        self.link1.bind("<Button>", lambda e: self.link("https://www.aeroaudio.eu/"))
        self.link1.bind("<Enter>", self.link_on_enter)
        self.link1.bind("<Leave>", self.link_on_leave)
        self.image = tk.PhotoImage(file = "Logo_AeroAudio.png").subsample(2)
        tk.Label(master = master, image = self.image, bg = "black").pack(side = tk.TOP)
        tk.Label(master = master, bg = "black").pack(pady = 10)

        tk.Label(master = master, text = "Enter IP address:", bg = "black", fg = "white").pack()
        self.ip_entry = tk.Entry(master = master)
        self.ip_entry.pack()
        self.connect_button = tk.Button(master = master, text = "Connect", command = self.connect, bg = "white", cursor = "hand2")
        self.connect_button.pack(pady = 5)
        self.connect_button.bind("<Enter>", self.connect_on_enter)
        self.connect_button.bind("<Leave>", self.connect_on_leave)
        tk.Label(master = master, bg = "black").pack(pady = 10)
        
        self.tcp_flag = tk.IntVar()
        self.dhd_flag = tk.IntVar()
        self.check_tcp = tk.Checkbutton(master = master, text="TCP", variable=self.tcp_flag, cursor = "hand2", bg = "black", fg = "white", activebackground = "black", activeforeground = "white", selectcolor = "grey26")
        self.check_tcp.pack()
        self.check_dhd = tk.Checkbutton(master = master, text="DHD", variable=self.dhd_flag, cursor = "hand2", bg = "black", fg = "white", activebackground = "black", activeforeground = "white", selectcolor = "grey26")
        self.check_dhd.pack()
        
        self.error_label = tk.Label(master = master, text = "Error: none",bg = "black", fg = "lime green")
        self.error_label.pack(pady = 10)
        tk.Label(master = master, text = "Exit:", bg = "black", fg = "white").pack()
        self.close_img = tk.PhotoImage(file="home.png").subsample(10)
        self.exit_button = tk.Button(master = master, image = self.close_img, command = self.close_ip, bg = "black", activebackground = "black", cursor = "hand2")
        self.exit_button.pack(pady = 3)
    
    def get_selected_mode(self):
        tcp = self.tcp_flag.get()
        dhd = self.dhd_flag.get()
        if tcp == 1 and dhd == 1:
            self.error = True
        elif tcp == 1 and dhd == 0:
            self.error = False
            return "TCP"
        elif dhd == 1 and tcp == 0:
            self.error = False
            return "DHD"
        else:
            self.error = False
    
    def connect(self):
        entered_ip = self.ip_entry.get()
        if entered_ip == None:
            self.error_label.config(text = "Error: enter an IP address", fg = "red")
        else:
            protocol = self.get_selected_mode()
            if self.error == False:
                if protocol == None:
                    self.error_label.config(text = "Select a protocol", fg = "red")
                else:
                    if self.callback(entered_ip, protocol):
                        self.master.destroy()
                    else:
                        self.error_label.config(text="Error: invalid IP address " + entered_ip, fg="red")
            else:
                self.error_label.config(text = "Only 1 protocol can be chosen", fg = "red")
            
    def link(self, url):
        webbrowser.get("C:/Program Files/Google/Chrome/Application/chrome.exe %s").open(url)
    
    def link_on_enter(self, e):
        self.link1.config(fg = "light blue")

    def link_on_leave(self, e):
        self.link1.config(fg = "aqua")
        
    def connect_on_enter(self, e):
        self.connect_button.config(bg = "SystemButtonFace", fg = "black")

    def connect_on_leave(self, e):
        self.connect_button.config(bg = "white", fg = "black")
        
    def close_ip(self):
        self.master.destroy()
        self.master.quit()

class DHDEntryWindow():
    def __init__(self, master, tcp_app):
        self.auto_flag = False
        self.auto_interval = 260
        self.mic1_button = False
        self.mic2_button = False
        self.mic3_button = False
        self.mic4_button = False
        self.connected = False
        self.tcp_app = tcp_app
        self.ip = self.tcp_app.server_address[0]
        self.master = master
        master.title("DHD MIC Control App")
        master.geometry("480x590")
        master.config(bg = "black")
        
        self.link1 = tk.Label(master = master, text = "AeroAudio.eu", bg = "black", fg = "aqua", cursor = "hand2")
        font = tkFont.Font(self.link1, self.link1.cget("font"))
        font.configure(underline = True)
        self.link1.configure(font = font)
        self.link1.bind("<Button>", lambda e: self.link("https://www.aeroaudio.eu/"))
        self.link1.bind("<Enter>", self.link_on_enter)
        self.link1.bind("<Leave>", self.link_on_leave)
        self.image = tk.PhotoImage(file = "Logo_AeroAudio.png").subsample(2)
        self.image_label = tk.Label(master = master, image = self.image, bg = "black")
        self.label = tk.Label(master = master, height = 2, bg = "black")
        
        self.mic1_label = tk.Label(master = master, text = "Current color Mic1:", bg = "black", fg = "white")
        self.color_mic1_label = tk.Label(master = master, padx = 15, pady = 5)
        
        self.mic2_label = tk.Label(master = master, text = "Current color Mic2:", bg = "black", fg = "white")
        self.color_mic2_label = tk.Label(master = master, padx = 15, pady = 5)
        
        self.mic3_label = tk.Label(master = master, text = "Current color Mic3:", bg = "black", fg = "white")
        self.color_mic3_label = tk.Label(master = master, padx = 15, pady = 5)
        
        self.mic4_label = tk.Label(master = master, text = "Current color Mic4:", bg = "black", fg = "white")
        self.color_mic4_label = tk.Label(master = master, padx = 15, pady = 5)
        
        self.auto_label = tk.Label(master = master, text = "Automatic disabled", bg = "black", fg = "white")
        self.auto_on = tk.Button(master = master, text = "Automatic on", command = self.toggle_auto, cursor = "hand2", bg = "red", activebackground = "red")
        self.auto_off = tk.Button(master = master, text = "Automatic off", command = self.stop_auto, cursor = "hand2", bg = "white")
        
        self.label_mic1 = tk.Label(master = master, text = "Mic1:", bg = "black", fg = "white")
        self.label_mic2 = tk.Label(master = master, text = "Mic2:", bg = "black", fg = "white")
        self.label_mic3 = tk.Label(master = master, text = "Mic3:", bg = "black", fg = "white")
        self.label_mic4 = tk.Label(master = master, text = "Mic4:", bg = "black", fg = "white")
        
        self.button_mic1_on = tk.Button(master = master, text = "On", command = self.toggle_mic1_on, bg = "white", activebackground = "white", width = 10, cursor = "hand2")
        self.button_mic2_on = tk.Button(master = master, text = "On", command = self.toggle_mic2_on, bg = "white", activebackground = "white", width = 10, cursor = "hand2")
        self.button_mic3_on = tk.Button(master = master, text = "On", command = self.toggle_mic3_on, bg = "white", activebackground = "white", width = 10, cursor = "hand2")
        self.button_mic4_on = tk.Button(master = master, text = "On", command = self.toggle_mic4_on, bg = "white", activebackground = "white", width = 10, cursor = "hand2")
        self.button_mic1_off = tk.Button(master = master, text = "Off", command = self.toggle_mic1_off, bg = "white", activebackground = "white", width = 10, cursor = "hand2")
        self.button_mic2_off = tk.Button(master = master, text = "Off", command = self.toggle_mic2_off, bg = "white", activebackground = "white", width = 10, cursor = "hand2")
        self.button_mic3_off = tk.Button(master = master, text = "Off", command = self.toggle_mic3_off, bg = "white", activebackground = "white", width = 10, cursor = "hand2")
        self.button_mic4_off = tk.Button(master = master, text = "Off", command = self.toggle_mic4_off, bg = "white", activebackground = "white", width = 10, cursor = "hand2")
        
        self.error_label = tk.Label(master = master, text = "Error: none", bg = "black", fg = "lime green")
        self.exit_label = tk.Label(master = master, text = "Exit:", bg = "black", fg = "white")
        self.close_img = tk.PhotoImage(file="home.png").subsample(10)
        self.exit_button = tk.Button(master = master, image = self.close_img, command = self.close_mic_control, bg = "black", activebackground = "black", cursor = "hand2")
        
        self.link1.grid(row = 0, column = 2, columnspan = 2)
        tk.Label(master = master, width = 2, height = 1, bg = "black").grid(row = 1, column = 0)
        self.image_label.grid(row = 2, column = 1, columnspan = 4)
        tk.Label(master = master, width = 2, bg = "black").grid(row = 3, column = 0)
        self.mic1_label.grid(row = 4, column = 1)
        self.color_mic1_label.grid(row = 5, column = 1)
        self.mic2_label.grid(row = 4, column = 2)
        self.color_mic2_label.grid(row = 5, column = 2)
        self.mic3_label.grid(row = 4, column = 3)
        self.color_mic3_label.grid(row = 5, column = 3)
        self.mic4_label.grid(row = 4, column = 4)
        self.color_mic4_label.grid(row = 5, column = 4)
        tk.Label(master = master, height = 1, bg = "black").grid(row = 6, column = 0)
        self.auto_label.grid(row = 7, column = 2, columnspan = 2)
        self.auto_on.grid(row = 8, column = 2)
        self.auto_off.grid(row = 8, column = 3)
        tk.Label(master = master, height = 1, bg = "black").grid(row = 9, column = 0)
        self.label_mic1.grid(row = 10, column = 1)
        self.label_mic2.grid(row = 10, column = 2)
        self.label_mic3.grid(row = 10, column = 3)
        self.label_mic4.grid(row = 10, column = 4)
        self.button_mic1_on.grid(row = 11, column = 1)
        self.button_mic2_on.grid(row = 11, column = 2)
        self.button_mic3_on.grid(row = 11, column = 3)
        self.button_mic4_on.grid(row = 11, column = 4)
        self.button_mic1_off.grid(row = 12, column = 1)
        self.button_mic2_off.grid(row = 12, column = 2)
        self.button_mic3_off.grid(row = 12, column = 3)
        self.button_mic4_off.grid(row = 12, column = 4)
        tk.Label(master = master, height = 2, bg = "black").grid(row = 13, column = 0)
        self.error_label.grid(row = 14, column = 2, columnspan = 2)
        self.exit_label.grid(row = 15, column = 2, columnspan = 2)
        self.exit_button.grid(row = 16, column = 2, columnspan = 2)
    
    def toggle_auto(self):
        self.auto_label.config(text = "Automatic enabled")
        self.color_mic1_label.config(bg = "red")
        self.color_mic2_label.config(bg = "red")
        self.color_mic3_label.config(bg = "red")
        self.color_mic4_label.config(bg = "red")
        self.auto_flag = True
        self.auto_change_mic()
        
    def auto_change_mic(self):
        if self.auto_flag:
            self.toggle_mic("AERO\03\00\11P\00\00\00\00\01")
            time.sleep(0.2)
            self.toggle_mic("AERO\03\00\11P\00\00\00\01\01")
            time.sleep(0.2)
            self.toggle_mic("AERO\03\00\11P\00\00\00\02\01")
            time.sleep(0.2)
            self.toggle_mic("AERO\03\00\11P\00\00\00\03\01")
            time.sleep(0.2)
            self.toggle_mic("AERO\03\00\11P\00\00\00\00\00")
            time.sleep(0.2)
            self.toggle_mic("AERO\03\00\11P\00\00\00\01\00")
            time.sleep(0.2)
            self.toggle_mic("AERO\03\00\11P\00\00\00\02\00")
            time.sleep(0.2)
            self.toggle_mic("AERO\03\00\11P\00\00\00\03\00")
            time.sleep(0.2)
            self.master.after(self.auto_interval, self.auto_change_mic)
            
    def stop_auto(self):
        self.auto_flag = False
        self.auto_label.config(text = "Automatic disabled")
        self.color_mic1_label.config(bg = "white")
        self.color_mic2_label.config(bg = "white")
        self.color_mic3_label.config(bg = "white")
        self.color_mic4_label.config(bg = "white")
    
    def close_mic_control(self):
        self.master.destroy()
        self.master.quit()
    
    def link(self, url):
        webbrowser.get("C:/Program Files/Google/Chrome/Application/chrome.exe %s").open(url)
    
    def link_on_enter(self, e):
        self.link1.config(fg = "light blue")

    def link_on_leave(self, e):
        self.link1.config(fg = "aqua")
    
    def toggle_mic1_on(self):
        self.mic1_button = True
        self.toggle_mic("AERO\03\00\11P\00\00\00\00\01")
        self.color_mic1_label.config(bg = "red")

    def toggle_mic1_off(self):
        self.mic1_button = False
        self.toggle_mic("AERO\03\00\11P\00\00\00\00\00")
        self.color_mic1_label.config(bg = "white")

    def toggle_mic2_on(self):
        self.mic2_button = True
        self.toggle_mic("AERO\03\00\11P\00\00\00\01\01")
        self.color_mic2_label.config(bg = "red")

    def toggle_mic2_off(self):
        self.mic2_button = False
        self.toggle_mic("AERO\03\00\11P\00\00\00\01\00")
        self.color_mic2_label.config(bg = "white")
            
    def toggle_mic3_on(self):
        self.mic3_button = True
        self.toggle_mic("AERO\03\00\11P\00\00\00\02\01")
        self.color_mic3_label.config(bg = "red")
        
    def toggle_mic3_off(self):
        self.mic3_button = False
        self.toggle_mic("AERO\03\00\11P\00\00\00\02\00")
        self.color_mic3_label.config(bg = "white")
        
    def toggle_mic4_on(self):
        self.mic4_button = True
        self.toggle_mic("AERO\03\00\11P\00\00\00\03\01")
        self.color_mic4_label.config(bg = "red")
        
    def toggle_mic4_off(self):
        self.mic4_button = False
        self.toggle_mic("AERO\03\00\11P\00\00\00\03\00")
        self.color_mic4_label.config(bg = "white")
        
    def toggle_mic(self, command):
        error = False
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        try:
            s.connect((self.ip, 2009))
            s.send(command.encode())
            s.close()
            error = False
        except Exception as e:
            error = True
            self.error_label.config(text = "Error: " + str(e), fg = "red")

        if not error:
            self.error_label.config(text = "Error: none", fg = "green")

class TCPApp:
    def __init__(self, master):
        self.auto_flag = False
        self.auto_interval = 260
        self.mic1_button = False
        self.mic2_button = False
        self.mic3_button = False
        self.mic4_button = False
        self.connected = False
        
        self.server_address = ("", 0)
        self.master = master
        master.title("TCP MIC Control App")
        master.geometry("480x590")
        master.config(bg = "black")
        
        self.link1 = tk.Label(master = master, text = "AeroAudio.eu", bg = "black", fg = "aqua", cursor = "hand2")
        font = tkFont.Font(self.link1, self.link1.cget("font"))
        font.configure(underline = True)
        self.link1.configure(font = font)
        self.link1.bind("<Button>", lambda e: self.link("https://www.aeroaudio.eu/"))
        self.link1.bind("<Enter>", self.link_on_enter)
        self.link1.bind("<Leave>", self.link_on_leave)
        self.image = tk.PhotoImage(file = "Logo_AeroAudio.png").subsample(2)
        self.image_label = tk.Label(master = master, image = self.image, bg = "black")
        
        self.mic1_label = tk.Label(master = master, text = "Current color Mic1:", bg = "black", fg = "white")
        self.color_mic1_label = tk.Label(master = master, padx = 15, pady = 5)
        
        self.mic2_label = tk.Label(master = master, text = "Current color Mic2:", bg = "black", fg = "white")
        self.color_mic2_label = tk.Label(master = master, padx = 15, pady = 5)
        
        self.mic3_label = tk.Label(master = master, text = "Current color Mic3:", bg = "black", fg = "white")
        self.color_mic3_label = tk.Label(master = master, padx = 15, pady = 5)
        
        self.mic4_label = tk.Label(master = master, text = "Current color Mic4:", bg = "black", fg = "white")
        self.color_mic4_label = tk.Label(master = master, padx = 15, pady = 5)
        
        self.auto_label = tk.Label(master = master, text = "Automatic disabled", bg = "black", fg = "white")
        self.auto_button = tk.Button(master = master, text = "Automatic", command = self.toggle_auto, width = 15, cursor = "hand2", bg = "red")
        self.auto_button.bind("<Enter>", self.auto_on_enter)
        self.auto_button.bind("<Leave>", self.auto_on_leave)
        self.stop_auto_button = tk.Button(master = master, text = "Stop Automatic", command = self.stop_auto, width = 15, cursor = "hand2", bg = "white")
        self.stop_auto_button.bind("<Enter>", self.stop_auto_on_enter)
        self.stop_auto_button.bind("<Leave>", self.stop_auto_on_leave)
        
        self.label_mic1 = tk.Label(master = master, text = "Mic1:", bg = "black", fg = "white")
        self.label_mic2 = tk.Label(master = master, text = "Mic2:", bg = "black", fg = "white")
        self.label_mic3 = tk.Label(master = master, text = "Mic3:", bg = "black", fg = "white")
        self.label_mic4 = tk.Label(master = master, text = "Mic4:", bg = "black", fg = "white")
        
        self.button_mic1_on = tk.Button(master = master, text = "On", command = self.toggle_mic1_on, bg = "white", activebackground = "white", width = 10, cursor = "hand2")
        self.button_mic1_off = tk.Button(master = master, text = "Off", command = self.toggle_mic1_off, bg = "white", activebackground = "white", width = 10, cursor = "hand2")
        self.button_mic2_on = tk.Button(master = master, text = "On", command = self.toggle_mic2_on, bg = "white", activebackground = "white", width = 10, cursor = "hand2")
        self.button_mic2_off = tk.Button(master = master, text = "Off", command = self.toggle_mic2_off, bg = "white", activebackground = "white", width = 10, cursor = "hand2")
        self.button_mic3_on = tk.Button(master = master, text = "On", command = self.toggle_mic3_on, bg = "white", activebackground = "white", width = 10, cursor = "hand2")
        self.button_mic3_off = tk.Button(master = master, text = "Off", command = self.toggle_mic3_off, bg = "white", activebackground = "white", width = 10, cursor = "hand2")
        self.button_mic4_on = tk.Button(master = master, text = "On", command = self.toggle_mic4_on, bg = "white", activebackground = "white", width = 10, cursor = "hand2")
        self.button_mic4_off = tk.Button(master = master, text = "Off", command = self.toggle_mic4_off, bg = "white", activebackground = "white", width = 10, cursor = "hand2")
            
        self.error_label = tk.Label(master = master, text = "Error: none", bg = "black", fg = "lime green")

        self.exit_label = tk.Label(master = master, text = "Exit:", bg = "black", fg = "white")
        self.close_img = tk.PhotoImage(file="home.png").subsample(10)
        self.exit_button = tk.Button(master = master, image = self.close_img, command = self.close_mic_control, bg = "black", activebackground = "black", cursor = "hand2")
        
        self.link1.grid(row = 0, column = 2, columnspan = 2)
        tk.Label(master = master, width = 2, height = 1, bg = "black").grid(row = 1, column = 0)
        self.image_label.grid(row = 2, column = 1, columnspan = 4)
        tk.Label(master = master, width = 2, bg = "black").grid(row = 3, column = 0)
        self.mic1_label.grid(row = 4, column = 1)
        self.color_mic1_label.grid(row = 5, column = 1)
        self.mic2_label.grid(row = 4, column = 2)
        self.color_mic2_label.grid(row = 5, column = 2)
        self.mic3_label.grid(row = 4, column = 3)
        self.color_mic3_label.grid(row = 5, column = 3)
        self.mic4_label.grid(row = 4, column = 4)
        self.color_mic4_label.grid(row = 5, column = 4)
        tk.Label(master = master, height = 1, bg = "black").grid(row = 6, column = 0)
        self.auto_label.grid(row = 7, column = 2, columnspan = 2)
        self.auto_button.grid(row = 8, column = 2)
        self.stop_auto_button.grid(row = 8, column = 3)
        tk.Label(master = master, height = 1, bg = "black").grid(row = 9, column = 0)
        self.label_mic1.grid(row = 10, column = 1)
        self.label_mic2.grid(row = 10, column = 2)
        self.label_mic3.grid(row = 10, column = 3)
        self.label_mic4.grid(row = 10, column = 4)
        self.button_mic1_on.grid(row = 11, column = 1)
        self.button_mic2_on.grid(row = 11, column = 2)
        self.button_mic3_on.grid(row = 11, column = 3)
        self.button_mic4_on.grid(row = 11, column = 4)
        self.button_mic1_off.grid(row = 12, column = 1)
        self.button_mic2_off.grid(row = 12, column = 2)
        self.button_mic3_off.grid(row = 12, column = 3)
        self.button_mic4_off.grid(row = 12, column = 4)
        tk.Label(master = master, height = 2, bg = "black").grid(row = 13, column = 0)
        self.error_label.grid(row = 14, column = 2, columnspan = 2)
        self.exit_label.grid(row = 15, column = 2, columnspan = 2)
        self.exit_button.grid(row = 16, column = 2, columnspan = 2)
        
    def check_and_assign_ip(self, entered_ip, protocol):
        if protocol == "TCP":
            if self.is_valid_ip(entered_ip):
                self.assign_ip(entered_ip)
                self.connected = True
                self.show_led_control()
                return True
            else:
                return False
        elif protocol == "DHD":
            self.assign_ip(entered_ip)
            self.connected = True
            self.open_dhd_entry_window()
            return True
        else:
            self.error_label.config(text = "Select a protocol", fg = "red")
            return False

    def open_ip_entry_window(self):
        ip_entry_window = tk.Toplevel(self.master)
        IPEntryWindow(ip_entry_window, self.check_and_assign_ip)
        
    def open_dhd_entry_window(self):
        dhd_entry_window = tk.Toplevel(self.master)
        DHDEntryWindow(dhd_entry_window, self)

    def show_led_control(self):
        if self.connected:
            self.master.deiconify() 
    
    def assign_ip(self, ip):
        self.server_address = (ip, 93)
        print(f"Assigned IP address: {ip}")
        self.error_label.config(text = "Error: none", fg = "green")
        time.sleep(0.2)
        self.toggle_mic("Off1")
        time.sleep(0.2)
        self.toggle_mic("Off2")
        time.sleep(0.2)
        self.toggle_mic("Off3")
        time.sleep(0.2)
        self.toggle_mic("Off4")

    def is_valid_ip(self, ip):
        try:
            socket.create_connection((ip, 93), timeout = 1)
            return True
        except (socket.error, socket.timeout):
            return False

    def link(self, url):
        webbrowser.get("C:/Program Files/Google/Chrome/Application/chrome.exe %s").open(url)
    
    def link_on_enter(self, e):
        self.link1.config(fg = "light blue")

    def link_on_leave(self, e):
        self.link1.config(fg = "aqua")

    def auto_on_enter(self, e):
        self.auto_button.config(bg = "white")

    def auto_on_leave(self, e):
        self.auto_button.config(bg = "red")
        
    def stop_auto_on_enter(self, e):
        self.stop_auto_button.config(bg = "red")

    def stop_auto_on_leave(self, e):
        self.stop_auto_button.config(bg = "white")
    
    def toggle_auto(self):
        self.auto_label.config(text = "Automatic enabled")
        self.color_mic1_label.config(bg = "red")
        self.color_mic2_label.config(bg = "red")
        self.color_mic3_label.config(bg = "red")
        self.color_mic4_label.config(bg = "red")
        self.auto_flag = True
        self.auto_change_mic()
    
    def stop_auto(self):
        self.auto_flag = False
        self.auto_label.config(text = "Automatic disabeld")
        self.color_mic1_label.config(bg = "white")
        self.color_mic2_label.config(bg = "white")
        self.color_mic3_label.config(bg = "white")
        self.color_mic4_label.config(bg = "white")
    
    def auto_change_mic(self):
        if self.auto_flag:
            self.toggle_mic("On1")
            time.sleep(0.2)
            self.toggle_mic("On2")
            time.sleep(0.2)
            self.toggle_mic("On3")
            time.sleep(0.2)
            self.toggle_mic("On4")
            time.sleep(0.2)
            self.toggle_mic("Off1")
            time.sleep(0.2)
            self.toggle_mic("Off2")
            time.sleep(0.2)
            self.toggle_mic("Off3")
            time.sleep(0.2)
            self.toggle_mic("Off4")
            time.sleep(0.2)
            self.master.after(self.auto_interval, self.auto_change_mic)
            
    def close_mic_control(self):
        self.master.destroy()
        self.master.quit()

    def toggle_mic1_on(self):
        self.mic1_button = True
        self.toggle_mic("On1")
        self.color_mic1_label.config(bg = "red")

    def toggle_mic1_off(self):
        self.mic1_button = False
        self.toggle_mic("Off1")
        self.color_mic1_label.config(bg = "white")

    def toggle_mic2_on(self):
        self.mic2_button = True
        self.toggle_mic("On2")
        self.color_mic2_label.config(bg = "red")

    def toggle_mic2_off(self):
        self.mic2_button = False
        self.toggle_mic("Off2")
        self.color_mic2_label.config(bg = "white")
            
    def toggle_mic3_on(self):
        self.mic3_button = True
        self.toggle_mic("On3")
        self.color_mic3_label.config(bg = "red")
        
    def toggle_mic3_off(self):
        self.mic3_button = False
        self.toggle_mic("Off3")
        self.color_mic3_label.config(bg = "white")
        
    def toggle_mic4_on(self):
        self.mic4_button = True
        self.toggle_mic("On4")
        self.color_mic4_label.config(bg = "red")
        
    def toggle_mic4_off(self):
        self.mic4_button = False
        self.toggle_mic("Off4")
        self.color_mic4_label.config(bg = "white")

    def toggle_mic(self, command):
        error = False
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.settimeout(0.1)
            s.connect(self.server_address)
            s.send(command.encode())
            s.recv(1024)
            s.close()
        except Exception as e:
            error = True
            if str(e) == "timed out":
                error = False
            else:
                self.error_label.config(text = "Error: " + str(e), fg = "red")

        if not error:
            self.error_label.config(text = "Error: none", fg = "green")
        
root = tk.Tk()
root.withdraw()
app = TCPApp(root)
app.open_ip_entry_window()
root.mainloop()